import { NormalizedProblem } from '../walk';
import { OasVersion, OasMajorVersion, Oas2RuleSet, Oas3RuleSet } from '../oas-types';
import type { NodeType } from '../types';
import type { DecoratorConfig, Plugin, PreprocessorConfig, Region, ResolveConfig, ResolvedApi, ResolvedConfig, ResolvedStyleguideConfig, RuleConfig, RuleSettings, ThemeRawConfig } from './types';
export declare const IGNORE_FILE = ".redocly.lint-ignore.yaml";
export declare const DEFAULT_REGION = "us";
export declare const DOMAINS: {
    us: string;
    eu: string;
};
export declare const AVAILABLE_REGIONS: Region[];
export declare class StyleguideConfig {
    rawConfig: ResolvedStyleguideConfig;
    configFile?: string | undefined;
    plugins: Plugin[];
    ignore: Record<string, Record<string, Set<string>>>;
    doNotResolveExamples: boolean;
    rules: Record<OasVersion, Record<string, RuleConfig>>;
    preprocessors: Record<OasVersion, Record<string, PreprocessorConfig>>;
    decorators: Record<OasVersion, Record<string, DecoratorConfig>>;
    private _usedRules;
    private _usedVersions;
    recommendedFallback: boolean;
    extendPaths: string[];
    pluginPaths: string[];
    constructor(rawConfig: ResolvedStyleguideConfig, configFile?: string | undefined);
    resolveIgnore(ignoreFile?: string): void;
    saveIgnore(): void;
    addIgnore(problem: NormalizedProblem): void;
    addProblemToIgnore(problem: NormalizedProblem): NormalizedProblem;
    extendTypes(types: Record<string, NodeType>, version: OasVersion): Record<string, NodeType>;
    getRuleSettings(ruleId: string, oasVersion: OasVersion): RuleSettings;
    getPreprocessorSettings(ruleId: string, oasVersion: OasVersion): RuleSettings;
    getDecoratorSettings(ruleId: string, oasVersion: OasVersion): RuleSettings;
    getUnusedRules(): {
        rules: string[];
        preprocessors: string[];
        decorators: string[];
    };
    getRulesForOasVersion(version: OasMajorVersion): Oas3RuleSet[] | Oas2RuleSet[];
    skipRules(rules?: string[]): void;
    skipPreprocessors(preprocessors?: string[]): void;
    skipDecorators(decorators?: string[]): void;
}
export declare class Config {
    rawConfig: ResolvedConfig;
    configFile?: string | undefined;
    apis: Record<string, ResolvedApi>;
    styleguide: StyleguideConfig;
    resolve: ResolveConfig;
    licenseKey?: string;
    region?: Region;
    theme: ThemeRawConfig;
    organization?: string;
    files: string[];
    constructor(rawConfig: ResolvedConfig, configFile?: string | undefined);
}
