import type { ProblemSeverity, UserContext } from '../walk';
import type { Oas3PreprocessorsSet, OasMajorVersion, Oas3DecoratorsSet, Oas2RuleSet, Oas2PreprocessorsSet, Oas2DecoratorsSet, Oas3RuleSet, OasVersion } from '../oas-types';
import type { NodeType } from '../types';
import { Location } from '../ref-utils';
import { SkipFunctionContext } from '../visitors';
export declare type RuleSeverity = ProblemSeverity | 'off';
export declare type RuleSettings = {
    severity: RuleSeverity;
};
export declare type PreprocessorSeverity = RuleSeverity | 'on';
export declare type RuleConfig = RuleSeverity | ({
    severity?: ProblemSeverity;
} & Record<string, any>);
export declare type PreprocessorConfig = PreprocessorSeverity | ({
    severity?: ProblemSeverity;
} & Record<string, any>);
export declare type DecoratorConfig = PreprocessorConfig;
export declare type StyleguideRawConfig = {
    plugins?: (string | Plugin)[];
    extends?: string[];
    doNotResolveExamples?: boolean;
    recommendedFallback?: boolean;
    rules?: Record<string, RuleConfig>;
    oas2Rules?: Record<string, RuleConfig>;
    oas3_0Rules?: Record<string, RuleConfig>;
    oas3_1Rules?: Record<string, RuleConfig>;
    preprocessors?: Record<string, PreprocessorConfig>;
    oas2Preprocessors?: Record<string, PreprocessorConfig>;
    oas3_0Preprocessors?: Record<string, PreprocessorConfig>;
    oas3_1Preprocessors?: Record<string, PreprocessorConfig>;
    decorators?: Record<string, DecoratorConfig>;
    oas2Decorators?: Record<string, DecoratorConfig>;
    oas3_0Decorators?: Record<string, DecoratorConfig>;
    oas3_1Decorators?: Record<string, DecoratorConfig>;
};
export declare type ApiStyleguideRawConfig = Omit<StyleguideRawConfig, 'plugins'>;
export declare type ResolvedStyleguideConfig = PluginStyleguideConfig & {
    plugins?: Plugin[];
    recommendedFallback?: boolean;
    extends?: void | never;
    extendPaths?: string[];
    pluginPaths?: string[];
};
export declare type PreprocessorsConfig = {
    oas3?: Oas3PreprocessorsSet;
    oas2?: Oas2PreprocessorsSet;
};
export declare type DecoratorsConfig = {
    oas3?: Oas3DecoratorsSet;
    oas2?: Oas2DecoratorsSet;
};
export declare type TypesExtensionFn = (types: Record<string, NodeType>, oasVersion: OasVersion) => Record<string, NodeType>;
export declare type TypeExtensionsConfig = Partial<Record<OasMajorVersion, TypesExtensionFn>>;
export declare type CustomRulesConfig = {
    oas3?: Oas3RuleSet;
    oas2?: Oas2RuleSet;
};
export declare type AssertionContext = Partial<UserContext> & SkipFunctionContext & {
    node: any;
};
export declare type AssertResult = {
    message?: string;
    location?: Location;
};
export declare type CustomFunction = (value: any, options: unknown, baseLocation: Location) => AssertResult[];
export declare type AssertionsConfig = Record<string, CustomFunction>;
export declare type Plugin = {
    id: string;
    configs?: Record<string, PluginStyleguideConfig>;
    rules?: CustomRulesConfig;
    preprocessors?: PreprocessorsConfig;
    decorators?: DecoratorsConfig;
    typeExtension?: TypeExtensionsConfig;
    assertions?: AssertionsConfig;
};
export declare type PluginStyleguideConfig = Omit<StyleguideRawConfig, 'plugins' | 'extends'>;
export declare type ResolveHeader = {
    name: string;
    envVariable?: undefined;
    value: string;
    matches: string;
} | {
    name: string;
    value?: undefined;
    envVariable: string;
    matches: string;
};
export declare type RawResolveConfig = {
    http?: Partial<HttpResolveConfig>;
    doNotResolveExamples?: boolean;
};
export declare type HttpResolveConfig = {
    headers: ResolveHeader[];
    customFetch?: Function;
};
export declare type ResolveConfig = {
    http: HttpResolveConfig;
};
export declare type Region = 'us' | 'eu';
export declare type AccessTokens = {
    [region in Region]?: string;
};
export declare type DeprecatedInRawConfig = {
    apiDefinitions?: Record<string, string>;
    lint?: StyleguideRawConfig;
    styleguide?: StyleguideRawConfig;
    referenceDocs?: Record<string, any>;
    apis?: Record<string, Api & DeprecatedInApi>;
} & DeprecatedFeaturesConfig;
export declare type Api = {
    root: string;
    styleguide?: ApiStyleguideRawConfig;
} & ThemeConfig;
export declare type DeprecatedInApi = {
    lint?: ApiStyleguideRawConfig;
} & DeprecatedFeaturesConfig;
export declare type ResolvedApi = Omit<Api, 'styleguide'> & {
    styleguide: ResolvedStyleguideConfig;
    files?: string[];
};
export declare type RawConfig = {
    apis?: Record<string, Api>;
    styleguide?: StyleguideRawConfig;
    resolve?: RawResolveConfig;
    region?: Region;
    organization?: string;
    files?: string[];
} & ThemeConfig;
export declare type FlatApi = Omit<Api, 'styleguide'> & Omit<ApiStyleguideRawConfig, 'doNotResolveExamples'>;
export declare type FlatRawConfig = Omit<RawConfig, 'styleguide' | 'resolve' | 'apis'> & Omit<StyleguideRawConfig, 'doNotResolveExamples'> & {
    resolve?: RawResolveConfig;
    apis?: Record<string, FlatApi>;
} & ThemeRawConfig;
export declare type ResolvedConfig = Omit<RawConfig, 'apis' | 'styleguide'> & {
    apis: Record<string, ResolvedApi>;
    styleguide: ResolvedStyleguideConfig;
};
declare type DeprecatedFeaturesConfig = {
    'features.openapi'?: Record<string, any>;
    'features.mockServer'?: Record<string, any>;
};
export declare type ThemeConfig = {
    theme?: ThemeRawConfig;
};
export declare type ThemeRawConfig = {
    openapi?: Record<string, any>;
    mockServer?: Record<string, any>;
};
export declare type RulesFields = 'rules' | 'oas2Rules' | 'oas3_0Rules' | 'oas3_1Rules' | 'preprocessors' | 'oas2Preprocessors' | 'oas3_0Preprocessors' | 'oas3_1Preprocessors' | 'decorators' | 'oas2Decorators' | 'oas3_0Decorators' | 'oas3_1Decorators';
export {};
