"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApprovalPayService = void 0;
const usuarioRepositories_1 = require("../../typeorm/repository/usuarioRepositories");
const pay_1 = require("../../queries/pay");
const verifyUser_1 = require("../../utils/verifyUser");
class ApprovalPayService {
    execute(sigla, trppCod, cereCod, valor, password, database) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const { message, error, status, userCod } = yield (0, verifyUser_1.verifyUserApproval)(sigla, password, database);
                if (error) {
                    console.log('====================================');
                    console.log({
                        message,
                        error,
                        status
                    });
                    console.log('====================================');
                    return ({
                        message,
                        error,
                        status
                    });
                }
                const autorizacaoCount = yield usuarioRepositories_1.UsuarioRepository.query(`
        USE [${database}]
        SELECT
          (
            SELECT
                COUNT(RETU_DATA)
            FROM
               REL_TRPP_USUA
            WHERE
                RETU_TRPP_COD = ${trppCod}
            AND
                RETU_USUA_COD = ${userCod}
          ) as countTrppUsua,
          PAGE_NUM_AUTORIZACOES
        FROM
          PARAMETROS_GERAIS
      `);
                if (Number(autorizacaoCount[0].PAGE_NUM_AUTORIZACOES) === Number(autorizacaoCount[0].countTrppUsua)) {
                    return {
                        error: true,
                        message: 'Parcela já autorizada',
                        status: 403
                    };
                }
                const selectValAprovAutoriza = yield usuarioRepositories_1.UsuarioRepository.query(`
          USE [${database}]
          SELECT
            USCR_VALOR_MAXIMO,
            USCR_AUTORIZA
          FROM
            USUARIO_CR
          WHERE
            USCR_USUA_COD = ${userCod}
          AND
            USCR_CERE_COD = ${cereCod}
        `);
                if (selectValAprovAutoriza[0].USCR_AUTORIZA === 'N') {
                    return {
                        error: true,
                        message: `Transação ${trppCod} - Usuario não é aprovador`,
                        status: 403
                    };
                }
                if (selectValAprovAutoriza[0].USCR_VALOR_MAXIMO < valor) {
                    return {
                        error: true,
                        message: `Transação ${trppCod} - usuario não pode aprovar um valor tão alto`,
                        status: 403
                    };
                }
                const queryApproval = (0, pay_1.approvalTrpp)(userCod + '', trppCod, database);
                yield usuarioRepositories_1.UsuarioRepository.query(queryApproval);
                return {
                    error: false,
                    message: `Transação ${trppCod} - aprovada com sucesso`,
                    status: 200
                };
            }
            catch (e) {
                return {
                    error: true,
                    message: `Internal Server Error: ${e}`,
                    status: 500
                };
            }
        });
    }
}
exports.ApprovalPayService = ApprovalPayService;
